from tkinter import *
from tkinter import messagebox
from core import function
import os
import subprocess
import hashlib
from gui.functions.basiclist import BasicFunctionList
from core import expression, function
from gui.widgets.smart import SmartButton, SmartCheckbutton, SmartEntry

class functionWindow(object):
    def __init__(self, function, callback=None):
        self.master = Toplevel()
        self.master.title("Function Editor")
        self.master.option_add("*Font", "courier 10")
        self.master.option_add("*Text.Font", "courier 10")
        self.master.option_add("*Button.Font", "courier 10")
        self.master.protocol("WM_DELETE_WINDOW", self.closeAction)    
        self.master.bind("<Key>", self.validateEntries)
        self.master.bind("<Control-v>", self.validateEntries)
        self.master.bind("<Button-1>", self.validateEntries)

        self.frm = Frame(self.master)
        function.window = self.master        
        
        # Save the function and its compiler object.
        self.function = function
        self.compiler = self.function.compiler
        self.callback = callback            
        self.makeWindow()
        self.readFunction()
        self.saveHash = self.makeHash()
        self.master.mainloop()

    def closeAction(self):
        newhash = self.makeHash()
        if newhash != self.saveHash:
            if messagebox.askyesno("Save", "Would you like to save?"):
                self.writeFunction()
        self.master.destroy()
        self.callback(self.function)
        
    def makeHash(self):
        hashdata = hashlib.md5()
        for field in self.ownfields:
            if(field is self.funcNameEntry or field is self.signatureEntry or field is self.funcHeaderEntry):
                hashdata.update(field.get().encode('utf-8'))
            else:
                hashdata.update(field.get(1.0,END).encode('utf-8'))
                
        # Encode the state of the includes list.
        includes = [func.compiler.name for func
                    in self.function_list.get_checked()] 
        hashdata.update(';'.join(includes).encode('utf-8'))
                
        #this isn't in ownfields because ownfields is only fields which enable eachother
        hashdata.update(self.constFrame.constEntry.get(1.0,END).encode('utf-8'))
        return hashdata.digest()
                
        
    def makeWindow(self):
        self.frm.grid()
        FIELD_SIZE = 80
        
        self.funcNameLabel = Label(self.frm, name='funcNameLabel', text='Function name')
        self.funcNameEntry = Entry(self.frm, name = 'funcNameEntry', width=FIELD_SIZE)

        self.signatureLabel = Label(self.frm, name='signatureLabel', text='Signature')
        self.signatureEntry = Entry(self.frm, name = 'signatureEntry', state=DISABLED, disabledbackground="gray", width=FIELD_SIZE)

        self.purposeLabel = Label(self.frm, name='purposeLabel', text='Purpose')
        self.purposeFrame = Frame(self.frm, bd=3, relief=GROOVE)
        self.purposeFrame.purposeEntry = Text(self.purposeFrame, name = 'purposeEntry', height = 3, width = FIELD_SIZE)
        self.purposeFrame.scroll = Scrollbar(self.purposeFrame)
        self.purposeFrame.purposeEntry.configure(yscrollcommand = self.purposeFrame.scroll.set)
        self.purposeFrame.scroll.config(command=self.purposeFrame.purposeEntry.yview)
       
        self.function_list = BasicFunctionList(self.frm, bd=5, relief=GROOVE, width=150, height=402, directory=self.compiler.directory)
        
        # Fill the function list with the valid functions from the current
        # directory.
        compilers = [func for func in
                     function.load_functions(self.compiler.directory)
                     if func.name != self.function.compiler.name]
        self.function_list.set(compilers)
        
        # Recheck any included functions that are visible in the list.
        checked_func_names = [compiler.name for compiler in
                              self.function.compiler.includes]
        for child in self.function_list.contents.children.values():
            if child.compiler.name in checked_func_names:
                child.check()
        
        self.function_list_title = Label(self.frm, name='function_list_title', text='Functions:')

        self.funcHeaderLabel = Label(self.frm, name='funcHeaderLabel', text='Function Header')
        self.funcHeaderEntry = Entry(self.frm, name = 'funcHeaderEntry', state=DISABLED, disabledbackground="gray", width=FIELD_SIZE)

        
        
        self.constLabel = Label(self.frm, name='constLabel', text='Constants')
        self.constFrame = Frame(self.frm, bd=3, relief=GROOVE)
        self.constFrame.constEntry = Text(self.constFrame, name = 'constEntry', height = 3, width = FIELD_SIZE)
        self.constFrame.scroll = Scrollbar(self.constFrame)
        self.constFrame.constEntry.configure(yscrollcommand = self.constFrame.scroll.set)
        self.constFrame.scroll.config(command=self.constFrame.constEntry.yview)

        self.exampleCallsLabel = Label(self.frm, name='exampleCallsLabel', text='Examples')
        
        self.exmplFrame = Frame(self.frm, bd=3, relief=GROOVE)
        self.exmplFrame.exampleEntry = Text(self.exmplFrame, name = 'exampleEntry',  state = DISABLED, background="gray", height = 3, width = FIELD_SIZE)
        self.exmplFrame.scroll = Scrollbar(self.exmplFrame)
        self.exmplFrame.exampleEntry.configure(yscrollcommand = self.exmplFrame.scroll.set)
        self.exmplFrame.scroll.config(command=self.exmplFrame.exampleEntry.yview)

        self.funcBodyLabel = Label(self.frm, name='funcBodyLabel', text='Function Body')
        self.funcBodyFrame = Frame(self.frm, bd=3, relief=GROOVE)
        self.funcBodyFrame.bodyEntry = Text(self.funcBodyFrame, name = 'bodyEntry',  state = DISABLED, background="gray", height = 10, width = FIELD_SIZE)
        self.funcBodyFrame.scroll = Scrollbar(self.funcBodyFrame)
        self.funcBodyFrame.bodyEntry.configure(yscrollcommand = self.funcBodyFrame.scroll.set)
        self.funcBodyFrame.scroll.config(command=self.funcBodyFrame.bodyEntry.yview)

        self.wallbtn = SmartCheckbutton(self.frm, name='wall', text="Compile strictly", checked=False)
        self.gorybtn2 = SmartCheckbutton(self.frm, name='gory', text="Detailed error messages", checked=False)

        self.compileButton = Button(self.frm, name ='compileButton', text='Compile, Test and Save', command=self.compileButton)

        self.saveButton = Button(self.frm, name ='saveButton', text='Save', command=self.saveButton)

        self.compileLabel = Label(self.frm, name ='compileLabel', text ='', anchor=W)
  

#tkinter stuff

        self.funcNameLabel.grid(row=0, column=0)
        self.funcNameEntry.grid(row=0, column=1, sticky=W, columnspan=4)
        self.function_list_title.grid(row=0, column=5, sticky='W')
        self.function_list.grid(row=1, column=5, rowspan=7, sticky='NS')
        
        self.signatureLabel.grid(row=1, column=0)
        self.signatureEntry.grid(row=1, column=1, sticky=W, columnspan=4)
        
        self.purposeLabel.grid(row=2, column=0)
        self.purposeFrame.purposeEntry.pack(side = LEFT)
        self.purposeFrame.purposeEntry.config(state=NORMAL)
        self.purposeFrame.scroll.pack(side = RIGHT, fill = Y)
        self.purposeFrame.grid(row=2, column = 1, sticky=W, columnspan=4)
        
        self.funcHeaderLabel.grid(row=3, column=0)
        self.funcHeaderEntry.grid(row=3, column=1, sticky=W, columnspan=4)
        
        self.constLabel.grid(row=4, column=0)
        self.constFrame.constEntry.pack(side = LEFT)
        self.constFrame.constEntry.config(state=NORMAL)
        self.constFrame.scroll.pack(side = RIGHT, fill = Y)
        self.constFrame.grid(row=4, column = 1, sticky=W, columnspan=4)

        self.exampleCallsLabel.grid(row=5, column=0)
        self.exmplFrame.exampleEntry.pack(side = LEFT)
        self.exmplFrame.scroll.pack(side = RIGHT, fill = Y)
        self.exmplFrame.grid(row=5, column = 1, sticky=W, columnspan=4)
                
        self.funcBodyLabel.grid(row=6, column=0)
        self.funcBodyFrame.bodyEntry.pack(side = LEFT)
        self.funcBodyFrame.scroll.pack(side = RIGHT, fill = Y)
        self.funcBodyFrame.grid(row=6, column=1, columnspan=4)

        self.wallbtn.grid(row=7, column=1, sticky=W)
        self.gorybtn2.grid(row=7, column=2, sticky=W)
        self.saveButton.grid(row=7, column=3, sticky=E)
        self.compileButton.grid(row=7, column=4, sticky=W)

        self.compileLabel.grid(row=8, columnspan=6)

        self.ownfields = [
            self.funcNameEntry,
            self.signatureEntry,
            self.purposeFrame.purposeEntry,
            self.funcHeaderEntry,
            self.exmplFrame.exampleEntry,
            self.funcBodyFrame.bodyEntry,
        ]
    def compileButton(self):
        self.saveHash = 0
        self.writeFunction(True)

    def saveButton(self):
        self.writeFunction(False)
        
    def writeFunction(self, shouldcompile=False):
        self.compileLabel['text'] = "Compiling..."
        data = dict(
            name = self.funcNameEntry.get(),
            signature = self.signatureEntry.get(),
            purpose = self.purposeFrame.purposeEntry.get(1.0, END),
            constants = self.constFrame.constEntry.get(1.0, END),
            header = self.funcHeaderEntry.get(),
            tests = self.exmplFrame.exampleEntry.get(1.0, END).rstrip('\n'),
            body = self.funcBodyFrame.bodyEntry.get(1.0, END).rstrip('\n')
        )

        newincludes = [function.compiler for function in self.function_list.get_checked()]
        self.function.compiler.includes = newincludes
        self.function.compiler.update(data)
        self.function.update()
        
        should_compile = self.validateEntries() == True and shouldcompile == True
        newhash = self.makeHash()
        if(newhash == self.saveHash):
            should_compile = False
        
        if should_compile:
            try:
                self.function.compiler.strict = self.wallbtn.is_checked()
                self.function.compiler.compile()
            except OSError as err:
                self.compileLabel['text'] = "Function saved. Error: %s" % err
            except subprocess.CalledProcessError as err:
                braceerror = self.function.compiler.check_braces(data['body'])
                if(braceerror is not None):
                    self.compileLabel['text'] = "Function saved. Could not be compiled : " + braceerror
                else:
                    if self.gorybtn2.is_checked():
                        result = err.output.decode('utf-8').splitlines()
                        result = '\n'.join([line.split(':', 1)[1].strip() for line in result])
                        messagebox.showwarning("Compile Failed!", result)
                    self.compileLabel['text'] = "Function saved. Could not be compiled."
            else:
                passed = self.makeResultWindow()
                if passed:
                    self.compileLabel['text'] = "Function saved and compiled. All tests passed."
                else:
                    self.compileLabel['text'] = "Function saved and compiled. Some tests failed."
        else:
            try:
                self.function.compiler.write_source_files()
                self.compileLabel['text'] = "Function saved."
            except AssertionError as err:
                self.compileLabel['text'] = str(err)
        
        # Update the function's name and other parameters for display in the
        # function list.
        if self.callback:
            self.callback(self.function)
        self.saveHash = self.makeHash()
                    
    def readFunction(self):
        # Enabled all the fields so that we actually set their values. 
        for field in self.ownfields:
            field.config(state=NORMAL, background="white")
            
        self.funcNameEntry.insert(0, self.compiler.name)
        self.signatureEntry.insert(0, self.compiler.signature)
        self.purposeFrame.purposeEntry.insert(INSERT, self.compiler.purpose)
        self.funcHeaderEntry.insert(0, self.compiler.header)
        self.exmplFrame.exampleEntry.insert(INSERT, self.compiler.tests)
        self.funcBodyFrame.bodyEntry.insert(INSERT, self.compiler.body)
        self.constFrame.constEntry.insert(INSERT, self.compiler.constants)
        
        # Some of the fields may be empty, so we need to revalidate everything.
        self.validateEntries()
            
    def validateEntries(self, event=None):
        self.compileLabel['text'] = ""
        entrycounter = 0
        blank = False
        for field in self.ownfields:
            field.config(state=NORMAL, background="white")
            if(blank == True):
                field.config(state=DISABLED, background="gray")
            else:
                if(field is self.funcNameEntry or field is self.signatureEntry or field is self.funcHeaderEntry):
                    if(len(field.get()) < 1):
                        blank = True
                else:
                    if(len(field.get(1.0,END)) < 2):
                        blank = True
                        
            entrycounter = entrycounter + 1
        
        # Return whether all fields are filled out.
        return not blank

    
    def center(self, window):
        """Will move a window to the center based on the screen"""
        sw = window.winfo_screenwidth()
        sh = window.winfo_screenheight()
        rw = window.winfo_reqwidth()
        rh = window.winfo_reqheight()
        xc = (sw - rw) / 2
        yc = (sh -rh) / 2
        window.geometry("+%d+%d" % (xc, yc))
        window.deiconify()
    
    def dialog(self, message):
        """A pop-up window used to display results"""
        win = Toplevel()
        win.title("test")
        Label(win,  text= message ).pack()  
        Button(win, text='OK', command=win.quit).pack()     
        win.protocol('WM_DELETE_WINDOW', win.quit)          

        win.focus_force()          
        win.grab_set()
        win.after(0,self.center,win)
        win.mainloop()           
        win.destroy()
        
    def makeResultWindow(self):
        if self.compiler.is_valid():
            results = self.compiler.run_tests()
            
            # Only display the results window if there was an error.
            if 'false' in list(zip(*results))[-1]:
                text = "Error: One or more of your example statements failed.\n\n"
                for i in results:
                    if i[-1] == 'false':
                        text += ">>> %s: %s <<<\n" % i
                    else:
                        text += "%s: %s\n" % i
                
                self.dialog(text)
                return False
            
            return True

    
