from tkinter import Label, Button, Checkbutton, Entry, IntVar, Frame, Canvas, Scrollbar
from tkinter import NORMAL, DISABLED, END, VERTICAL

from gui.widgets.highlightable import HighlightableList, HighlightableRow
from gui.widgets.smart import SmartButton, SmartCheckbutton
from core.function import FunctionCompiler, ExternalCompiler

from gui.functions.functionrow import Function

class BasicFunctionList(HighlightableList):
    def __init__(self, *args, **kwargs):
        self.directory = kwargs.pop('directory', None)
        
        super(BasicFunctionList, self).__init__(*args, **kwargs)
                
        # Layout the list and the buttons
        self.container.grid(row=0, column=0, columnspan=3)

    def get_checked(self):
        """Returns the functions that are checked."""
        checked = []
        for child in self.contents.children.values():
            try:
                if child.is_checked():
                    checked.append(child)
            
            # Some children may not support the ``is_checked`` method, so we
            # gracefully ignore them.
            except:
                pass
        
        return checked
    
    def clean(self, rows):
        """Allows plain old compiler objects to be passed to the append,
        prepend, set, etc. methods by automagically creating the
        Function objects for them. Yes, this *is* super awesome.
        
        Also forces the rows to be sorted by name. I know that this
        makes append and prepend do exactly the same thing, but tough.
        
        """
        rows = super(BasicFunctionList, self).clean(rows)
        
        cleaned_rows = []
        for row in rows:
            if isinstance(row, (FunctionCompiler, ExternalCompiler)):
                row = Function(self.contents, compiler=row)
                self.bind_row(row)
            if(row.is_valid()):
                cleaned_rows.append(row)
        
        # Sort the compilers by name.
        cleaned_rows.sort(key=lambda x: x.compiler.name)
        
        return cleaned_rows
    
    def get_unchecked(self):
        """Returns the functions that are unchecked."""
        unchecked = []
        for child in self.contents.children.values():
            try:
                if not child.is_checked():
                    unchecked.append(child)
            
            # Some children may not support the ``is_checked`` method, so we
            # gracefully ignore them.
            except:
                pass
        
        return unchecked
    
    def update(self, function):
        # Update the functions appearance.
        function.update()
        
        # Lay out all of the functions again, binding all rows, and setting
        # initial backgrounds.
        self.layout()
