from tkinter import Label, Button, Checkbutton, Entry, IntVar, Frame, Canvas, Scrollbar
from tkinter import NORMAL, DISABLED, END, VERTICAL

from gui.widgets.highlightable import HighlightableList, HighlightableRow
from gui.widgets.smart import SmartButton, SmartCheckbutton
from core.function import FunctionCompiler, ExternalCompiler

class Function(HighlightableRow):
    def __init__(self, *args, **kwargs):
        self.compiler = kwargs.pop('compiler')
        
        super(Function, self).__init__(*args, **kwargs)
                
        # Create and layout the widget children.
        self.checkbutton = SmartCheckbutton(self, checked=False)
        self.label = Label(self) # We set the text later.
        self.checkbutton.grid(row=0, column=0, sticky='W')
        self.label.grid(row=0, column=1, sticky='W')
        
        # Set the widgets that should be bound to click events (i.e. clicking on
        # these should highlight the row).
        self.widgets_to_bind = [self, self.label]
        
        # Update the appearance of the sub-widgets according to the state of the
        # function compiler instance.
        self.update()

    def should_layout(self):
        """Only lay out this function if it's compiler has a name."""
        return (bool(self.compiler.name) and
                super(Function, self).should_layout())
    
    def update(self):
        """Updates the appearance of the widget based on the *current*
        status of the compiler instance."""
        
        # NOTE: We append 1000 spaces to the name, so that row (and it's
        # background) fills the entire box. Note that this doesn't work without
        # the period at the end which is crazy in and of itself.
        text = self.compiler.name + ' ' * 1000 + '.'
        enabled = True
        color = 'Black'
        
        # If the function isn't editable, set the text color to gray.
        # TODO: Make the text italic too or say (external) or something...
        if not self.compiler.is_editable():
            color = 'Gray'

        # If the function isn't valid, disable the checkbox and set the text
        # color to red.
        if not self.compiler.is_valid():
            enabled = False
            color = 'Red'
        
        # Apply the styles.
        self.label.config(fg=color, text=text)
        if enabled:
            self.checkbutton.enable()
        else:
            self.checkbutton.disable()
    
#    def edit(self):
#        if not widget.is_editable():
#            raise TypeError('This function is not editable.')
#
#        # TEMP: Hacky... Fix functionGUI to take a compiler object.
#        import functionGUI
#        # Also this doesn't work...
#        functionGUI.functionGUI(self.directory, self.editbox, self)
#        
#        # TODO: Does this wait until the other window is closed? Or does this
#        # need to be a callback.
#        self.update()
#
    ################
    # PASSTHROUGHS #
    ################
    window = None
    def __getattr__(self, name):
        """Pass through any unmatched lookups to the checkbutton. This
        means you can call ``is_checked`` and ``disable`` on the
        Function instance, and have those commands passed through to the
        checkbutton.
        
        """
        return getattr(self.checkbutton, name)
    
    def is_valid(self):
        return self.compiler.is_valid()
    
    def is_editable(self):
        return self.compiler.is_editable()

    def delete(self):
        self.compiler.delete()
