from tkinter import Entry, Button, Checkbutton, IntVar
from tkinter import NORMAL, DISABLED, END

class SmartEntry(Entry):
    def clear(self):
        self.delete(0, END)
    
    def append(self, value):
        self.insert(END, value)
    
    def prepend(self, value):
        self.insert(0, value)
    
    def set(self, value):
        self.clear()
        self.append(value)
    
    def __str__(self):
        return self.get()

class SmartButton(Button):
    def __init__(self, *args, **kwargs):
        # Add a shortcut argument to disable the widget.
        if not kwargs.pop('enabled', True):
            kwargs['state'] = DISABLED

        super(SmartButton, self).__init__(*args, **kwargs)
    
    def enable(self):
        self.config(state=NORMAL)
    
    def disable(self):
        self.config(state=DISABLED)
    
    def is_enabled(self):
        return self['state'] != DISABLED
        
class SmartCheckbutton(Checkbutton):
    def __init__(self, *args, **kwargs):
        # Add a shortcut argument to disable the widget.
        if not kwargs.pop('enabled', True):
            kwargs['state'] = DISABLED
        
        # Add an argument to set the default button state.
        checked = kwargs.pop('checked', True)
        self._checked = IntVar(value=int(checked))
        
        kwargs['variable'] = self._checked
        super(SmartCheckbutton, self).__init__(*args, **kwargs)
        
    def check(self):
        self.select()
    
    def uncheck(self):
        self.deselect()

    def is_checked(self):
        return bool(self._checked.get())    
    
    def enable(self):
        self.config(state=NORMAL)
    
    def disable(self):
        self.config(state=DISABLED)
    
    def is_enabled(self):
        return self['state'] != DISABLED
    
    def __bool__(self):
        return self.is_checked()

