from tkinter import *
from tkinter import messagebox
from tkinter import filedialog
from core import expression, function
import functionGUI
from functionGUI import functionWindow
import os
import sched
import utils

# Experimental Subclasses
from gui.widgets.smart import SmartButton, SmartCheckbutton, SmartEntry
from gui.functions.list import FunctionList

class MainWindow(object):
    def __init__(self, master):
        self.frm = Frame(master)
        
        prefs = utils.load_preferences()
        try:
            directory = prefs['directory']
            assert os.path.exists(directory)
        except:
            directory = os.getcwd()
        self.directory = os.path.abspath(directory)
        self.frm.grid()
        self.makeWindow()
        
    def makeWindow(self):
        self.frm.grid()

        self.dirlabel = Label(self.frm, name='dirlabel', text='Current directory: %s' % self.directory)
        
        
        self.lbl1 = Label(self.frm, name='lbl1', text='Expression input:')
        self.entry1 = SmartEntry(self.frm, name='entry', width=70)
        
        # Create some buttons and checkboxes and stuff.
        self.runbtn = Button(self.frm, name='runbtn', text='Execute', command=self.runExpr)
        self.wallbtn = SmartCheckbutton(self.frm, name='wall', text="Compile strictly", checked=False)
        self.gorybtn = SmartCheckbutton(self.frm, name='gory', text="Detailed error messages", checked=False)
        self.aboutbtn = Button(self.frm, name='abtbtn', text='?', command=self.aboutbox)
        self.dirbtn = Button(self.frm, name='dirbtn', text='Change directory',
                             command=self.changeDir)
        
        self.function_list = FunctionList(self.frm, bd=5, relief=GROOVE, width=200, directory=self.directory)
        self.function_list_title = Label(self.frm, name='function_list_title', text='Functions:')

        self.textfr = Frame(self.frm)
        self.textfr.config(bd=5, relief=GROOVE)
        self.textfr.outputBox = Text(self.textfr, height = 15, width = 100)
        self.textfr.grid()
        self.textfr.scroll = Scrollbar(self.textfr)
        self.textfr.outputBox.configure(yscrollcommand = self.textfr.scroll.set)
        self.textfr.scroll.config(command=self.textfr.outputBox.yview)
        
        self.dirlabel.grid(row=0, sticky=W, columnspan = 2)
        self.dirbtn.grid(row=0, column = 2, sticky=W+E)
        self.lbl1.grid(row = 1, sticky=W)
        self.entry1.grid(row = 2, sticky=E+W, columnspan = 2)
        self.runbtn.grid(row = 2, column = 2, sticky=W+E)
        self.wallbtn.grid(row=1, column=1, sticky=W)
        self.gorybtn.grid(row=1, column=2, sticky=W)
       
        self.textfr.grid(row = 4, sticky=NS, columnspan = 3)
        self.textfr.outputBox.pack(side = LEFT, fill = Y)
        self.textfr.scroll.pack(side = RIGHT, fill = Y)
        self.function_list.grid(row=1, column=3, rowspan=8, columnspan=2, sticky='NS')
        self.function_list_title.grid(row=0, column=3, sticky='W')
        self.aboutbtn.grid(row=0, column=4, sticky=N+E)	
        
        self.populate_checkboxes()

    def aboutbox(self):
        win = Toplevel()
        win.title("About")
        photo = PhotoImage(file="gui/aboutbox.gif")
        Label(win,  image=photo ).pack()  
        Button(win, text='OK', command=win.quit).pack()     
        win.protocol('WM_DELETE_WINDOW', win.quit)          
        win.focus_force()          
        win.grab_set()
        win.mainloop()           
        win.destroy()
		
    def runExpr(self):
        value = self.entry1.get()
        
        # Grab the compiler objects off of the included functions.
        includes = [function.compiler for function in
                    self.function_list.get_checked()]
        
        # Create the expression compiler and evaluate it.
        compiler = expression.ExpressionCompiler(
            value,
            strict = self.wallbtn.is_checked(),
            gory = self.gorybtn.is_checked(),
            includes = includes,
        )
        result = compiler.evaluate()

        # Print the results to the interface.
        self.textfr.outputBox.delete('1.0', END)
        self.textfr.outputBox.insert(END, result)

    def changeDir(self): 
        directory = filedialog.askdirectory(title='Select a directory',
                                            mustexist=True)
        
        if directory:
            self.directory = os.path.abspath(directory)
            self.dirlabel['text'] = 'Current directory: %s' % self.directory
            
            # Save the directory to the preferences.
            utils.save_preferences({'directory': self.directory})
            
            # Yay! The directory is valid.
            self.function_list.unhighlight()
            self.function_list.directory = self.directory
            self.populate_checkboxes()
            

    
    def populate_checkboxes(self):
        compilers = function.load_functions(self.directory)
        self.function_list.set(compilers)

def theGUI():
    root = Tk()
    root.title("Helper++")
    root.option_add("*Font", "courier 10")
    root.option_add("*Text.Font", "courier 10")
    root.option_add("*Button.Font", "courier 10")
    showtime = MainWindow(root)
    root.mainloop()
    
theGUI()
