import json
import os
import sys

ROOT = os.path.realpath(os.path.dirname(__file__))
NAME = 'HelperPlusPlus'
NAMESPACE = 'com.480xp.helperplusplus'

def get_data_folder():
    """Gets the path to the preference folder."""
    if sys.platform == 'darwin':
        return os.path.expanduser("~/Library/Preferences")
    elif sys.platform == 'linux2':
        return os.path.expanduser("~/.%s" % NAME)
    elif sys.platform in ('win32', 'cygwin'):
        return os.path.join(os.environ['APPDATA'], NAME)

def get_data_file():
    """Gets the path to the preference file."""
    return os.path.join(get_data_folder(), "%s.txt" % NAMESPACE)

def save_preferences(prefs):
    """Tries to save the preferences, returns True or False, indicating
    whether the preferences were saved."""
    assert isinstance(prefs, dict)
    
    try:
        if not os.path.exists(get_data_folder()):
            os.makedirs(get_data_folder())
        
        with open(get_data_file(), 'w') as fp:
            json.dump(prefs, fp)
            return True
    except:
        # We ran into an errror, guess there won't be any pref saving today.
        return False

def load_preferences():
    """Tries to return the saved preferences. If there's an error,
    returns None."""
    try:
        with open(get_data_file(), 'r') as fp:
            prefs = json.load(fp)
            assert isinstance(prefs, dict)
            return prefs
    except:
        # We encountered some error. Oh well.
        return {}
